﻿using Bridge.Html5;
using FrontendRouting;
using System;

namespace LearnBridgeNet.Views
{
    [FeRoute("/logon.html")]
    internal class LogonView : FeRoutingView
    {
        private readonly HTMLElement _root;
        private readonly HTMLLabelElement _lbl;
        private readonly HTMLInputElement _chk;

        public LogonView()
        {
            _root = new HTMLDivElement();
            var ti = new HTMLHeadingElement(HeadingType.H1) { TextContent = "Logon Page" };
            var b = new HTMLInputElement { Type = InputType.Button, Value = "Login" };

            _lbl = new HTMLLabelElement { Hidden = true };
            _chk = new HTMLInputElement { Type = InputType.Checkbox };
            _lbl.AppendChild(_chk);
            _lbl.AppendChild(new Text("Leave check"));

            b.AddEventListener(EventType.Click, () =>
            {
                XState.IsLogin = true;
                ReturnToBeforeLoginPage();
            });

            _root.AppendChild(ti);
            _root.AppendChild(b);
            _root.AppendChild(_lbl);
        }

        private static void ReturnToBeforeLoginPage()
        {
            var returnUrl = Window.Location.Search.Substring(1);
            if (string.IsNullOrWhiteSpace(returnUrl)) returnUrl = "/";
            FeRoutingManager.Go(returnUrl, true);
        }

        public override bool BeforeEnter(FeRoutingView currentView)
        {
            if (XState.IsLogin)
            {
                ReturnToBeforeLoginPage();
                return false;
            }
            return base.BeforeEnter(currentView);
        }

        public override void Enter(Node parent)
        {
            App.NavigateBar.Hidden = true;
            _lbl.Hidden = true;
            _chk.Checked = false;
            parent.AppendChild(_root);
        }

        public override bool BeforeLeave(Action continueAction)
        {
            if (_lbl.Hidden)
            {
                _lbl.Hidden = false;
                return false;
            }
            if (_chk.Checked)
            {
                continueAction();
                return true;
            }
            return false;
        }

        public override void Leave()
        {
            _root.Remove();
            App.NavigateBar.Hidden = false;
        }
    }
}